<?php
/**
 * Class to handle all custom API endpoints for OTP
 */

if ( !defined( 'ABSPATH' ) )
	exit;

if ( !class_exists( 'ewdotpAPIHandler' ) ) {
class ewdotpAPIHandler {

	/**
	 * Hook in to register apis
	 * @since 1.0.0
	 */
	public function __construct() {

		// Add in all of the custom endpoints necessary
		add_action( 'rest_api_init', array( $this, 'register_api_endpoints' ) );

		// Add in filters to add the API settings panel to the different FSP plugins
		add_filter( 'ewd_otp_settings_page', array( $this, 'add_plugin_settings_panel' ) );

		add_action( 'admin_enqueue_scripts', 	array( $this, 'enqueue_admin_assets' ) );
	}

	/**
	 * Call plugin-specific API registrations if conditions met
	 * @since 1.0.0
	 */
	public function register_api_endpoints() {

		//Add in an endpoint to validate the API connection on account save
		$args = array(
			'methods' 				=> 'GET',
			'callback' 				=> array( $this, 'handle_validation_get_request' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'ewd-premium-helper/v1', '/validate/', $args );

		//Add in an endpoint where any our app will send the device token for push notifications
		$args = array(
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'handle_push_notification_device_token' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'ewd-premium-helper/v1', '/registerpushnotisub/', $args );

		if ( $this->ewd_otp_endpoints_valid() ) { $this->register_order_tracking_api_endpoints(); }
	}

	/**
	 * Determine whether order tracking apis should be activated
	 * @since 1.0.0
	 */
	public function ewd_otp_endpoints_valid() {
		global $ewd_otp_controller;

		if ( empty( $ewd_otp_controller ) or ! is_object( $ewd_otp_controller ) ) { return false; }

		if ( ! $ewd_otp_controller->permissions->check_permission( 'api_usage' ) ) { return false; }

		return true;
	}

	/**
	 * Register order tracking required apis
	 * @since 1.0.0
	 */
	public function register_order_tracking_api_endpoints() {

		//Add in an endpoint for getting a list of IDs, created times and statuses for all orders within a recent time frame
		$args = array(
			'methods' 				=> 'GET',
			'callback' 				=> array( $this, 'handle_orders_get_request' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'order-tracking/v1', '/orders/', $args );

		//Add in an endpoint for getting a list of valid order statuses
		$args = array(
			'methods' 				=> 'GET',
			'callback' 				=> array( $this, 'handle_order_statuses_get_request' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'order-tracking/v1', '/order-statuses/', $args );

		//Add in an endpoint for getting a list of valid order locations
		$args = array(
			'methods' 				=> 'GET',
			'callback' 				=> array( $this, 'handle_order_locations_get_request' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'order-tracking/v1', '/order-locations/', $args );

		//Add in an endpoint for getting specific orders by ID
		$args = array(
			'methods' 				=> 'GET',
			'callback' 				=> array( $this, 'handle_order_get_request' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'order-tracking/v1', '/order/(?P<id>\d+)', $args );

		//Add in an endpoint for setting a status for a specific order
		$args = array(
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'handle_order_set_status' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'order-tracking/v1', '/set-order-status/(?P<id>\d+)', $args );

		//Add in an endpoint for setting a location for a specific order
		$args = array(
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'handle_order_set_location' ),
			'permission_callback' 	=> '__return_true',
		);
		register_rest_route( 'order-tracking/v1', '/set-order-location/(?P<id>\d+)', $args );
	}

	/**
	 * Verify whether a submitted API key is valid
	 * @since 1.0.0
	 */
	public function verify_api_key( $api_key ) {
		global $ewd_otp_controller;

		$valid_api_keys = ewd_otp_decode_infinite_table_setting( $ewd_otp_controller->settings->get_setting( 'ewd-api-keys' ) );

		foreach ( $valid_api_keys as $valid_api_key ) {

			if ( $api_key == $valid_api_key->api_key ) { return true; }
		}

		return false;
	}

	/**
	 * Verify whether a submitted API key is valid
	 * @since 1.0.0
	 */
	public function verify_write_api_key_access( $api_key ) {
		global $ewd_otp_controller;

		$valid_api_keys = ewd_otp_decode_infinite_table_setting( $ewd_otp_controller->settings->get_setting( 'ewd-api-keys' ) );

		foreach ( $valid_api_keys as $valid_api_key ) {

			if ( $api_key == $valid_api_key->api_key and $valid_api_key->access == 'update' ) { return true; }
		}

		return false;
	}

	/**
	 * Return a success or failure message to validate an API key
	 * @since 1.0.0
	 */
	public function handle_validation_get_request( $request ) {

		$success = $this->verify_api_key( $request->get_param( 'apiKey' ) );

		if ( ! $success ) {

			return array(
				'success' => false,
				'message' => __( 'API key is not valid. Please be sure to copy it exactly from your plugin settings page.', 'ewd-premium-helper' )
			);
		}

		return array( 'success' => true, 'message' => __( 'Settings have been successfully saved.', 'ewd-premium-helper' ) );
	}

	/**
	 * Get a list of all orders with names, statuses, order time and IDs
	 * @since 1.0.0
	 */
	public function handle_orders_get_request ( $request ) {
		global $ewd_otp_controller;

		if ( ! $this->verify_api_key( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_invalid_api_key();
		}

		$args = array( 
			'orders_per_page' 	=> -1,
			'orderby'			=> 'date',
			'order'				=> 'DESC'
		);

		if ( ! empty( $request->get_param( 'start_date_time' ) ) ) {

			$args['after'] = $request->get_param( 'start_date_time' );
		}

		if ( ! empty( $request->get_param( 'end_date_time' ) ) ) {

			$args['before'] = $request->get_param( 'end_date_time' );
		}

		if ( ! empty( $request->get_param( 'status' ) ) ) {

			$args['status'] = $request->get_param( 'status' );
		}

		if ( ! empty( $request->get_param( 'location' ) ) ) {

			$args['location'] = $request->get_param( 'location' );
		}

		$orders = $ewd_otp_controller->order_manager->get_matching_orders( $args );

		$email = ! empty( $request->get_param( 'email' ) ) ? sanitize_text_field( $request->get_param( 'email' ) ) : false;
		$phone = ! empty( $request->get_param( 'phone' ) ) ? sanitize_text_field( $request->get_param( 'phone' ) ) : false;
		$name = ! empty( $request->get_param( 'name' ) ) ? sanitize_text_field( $request->get_param( 'name' ) ) : false;

		$return_data = array();
		foreach ( $orders as $order ) {

			if ( ! empty( $email ) and strpos( $order->email, $email ) === false ) { continue; }
			if ( ! empty( $phone ) and strpos( $order->phone_number, $phone ) === false ) { continue; }
			if ( ! empty( $name ) and strpos( $order->name, $name ) === false ) { continue; }

			$data = array(
				'id' 				=> $order->id,
				'date' 				=> $order->status_updated_fmtd,
				'status'			=> $order->status,
				'location'			=> $order->location,
				'number'			=> $order->number,
				'name'				=> $order->name,
				'phone'				=> $order->phone_number,
				'email'				=> $order->email,
				'public_note'		=> $order->notes_public,
				'private_note'		=> $order->notes_private,
				'customer_note'		=> $order->customer_notes,
				'customer'			=> $ewd_otp_controller->customer_manager->get_customer_field( 'name', $order->customer ),
				'customer_id'		=> $order->customer,
				'sales_rep'			=> $ewd_otp_controller->sales_rep_manager->get_sales_rep_field( 'name', $order->customer ),
				'sales_rep_id'		=> $order->sales_rep,
				'custom_fields'		=> implode( ', ', $order->custom_fields ),
				'payment_price'		=> $order->payment_price,
				'payment_completed'	=> $order->payment_completed,
			);

			$return_data[] = $data;
		}

		return array( 'success' => true, 'data' => $return_data );
	}

	/**
	 * Get all valid order statuses
	 * @since 1.0.0
	 */
	public function handle_order_statuses_get_request( $request ) {
		global $ewd_otp_controller;

		if ( ! $this->verify_api_key( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_invalid_api_key();
		}

		$statuses = ewd_otp_decode_infinite_table_setting( $ewd_otp_controller->settings->get_setting( 'statuses' ) );

		return array( 'success' => true, 'data' => $statuses );
	}

	/**
	 * Get all valid order locations
	 * @since 1.0.0
	 */
	public function handle_order_locations_get_request( $request ) {
		global $ewd_otp_controller;

		if ( ! $this->verify_api_key( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_invalid_api_key();
		}

		$locations = ewd_otp_decode_infinite_table_setting( $ewd_otp_controller->settings->get_setting( 'locations' ) );

		return array( 'success' => true, 'data' => $locations );
	}

	/**
	 * Get a specific order based on its ID
	 * @since 1.0.0
	 */
	public function handle_order_get_request( $request ) {

		if ( ! $this->verify_api_key( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_invalid_api_key();
		}
		
		$order = new ewdotpOrder();
		
		if ( ! $order->load_order_from_id( intval( $request->get_param( 'id' ) ) ) ) {
			
			return array( 'success' => false, 'Invalid Order ID' );
		}

		$order->load_order_status_history();

		return array( 'success' => true, 'data' => $order );
	}

	/**
	 * Set the status for an order
	 * @since 1.0.0
	 */
	public function handle_order_set_status( $request ) {
		
		if ( ! $this->verify_api_key( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_invalid_api_key();
		}

		if ( ! $this->verify_write_api_key_access( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_insufficient_api_key_privileges();
		}

		if ( empty( $request->get_param( 'id' ) ) or empty( $request->get_param( 'postStatus' ) ) ) { return; }

		$order = new ewdotpOrder();
		
		if ( ! $order->load_order_from_id( intval( $request->get_param( 'id' ) ) ) ) {

			return array( 'success' => false, 'Invalid Order ID' );
		}
		
		$post_status = sanitize_text_field( $request->get_param( 'postStatus' ) );

		$order->set_status( $post_status );
		
		return array( 'success' => true, 'message' => __( 'Order has been successfully updated.', 'ewd-premium-helper' ) );
	}

	/**
	 * Set the location for an order
	 * @since 1.0.0
	 */
	public function handle_order_set_location( $request ) {
		
		if ( ! $this->verify_api_key( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_invalid_api_key();
		}

		if ( ! $this->verify_write_api_key_access( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_insufficient_api_key_privileges();
		}

		if ( empty( $request->get_param( 'id' ) ) or empty( $request->get_param( 'postLocation' ) ) ) { return; }

		$order = new ewdotpOrder();
		
		if ( ! $order->load_order_from_id( intval( $request->get_param( 'id' ) ) ) ) {

			return array( 'success' => false, 'Invalid Order ID' );
		}
		
		$post_location = sanitize_text_field( $request->get_param( 'postLocation' ) );

		$order->set_location( $post_location );
		
		return array( 'success' => true, 'message' => __( 'Order has been successfully updated.', 'ewd-premium-helper' ) );
	}

	/**
	 * Save token to database sent from app
	 * */
	public function handle_push_notification_device_token( $request ) {

		if ( ! $this->verify_api_key( $request->get_param( 'apiKey' ) ) ) {

			return $this->return_invalid_api_key();
		}

		if ( ! rtb_ultimate_active() and ! fdm_ultimate_active() ) {

			return $this->return_no_ultimate_active();
		}

		$token = $request->get_param( 'token' );

		$messages = array();

		if ( rtb_ultimate_active() ) {

			$messages[] = ! empty( get_option( 'rtb-license-key', false ) ) ? $this->add_notification_token( $token, 'rtu', get_option( 'rtb-license-key' ) ) : __( 'Invalid Restaurant Reservations Ultimate license key.', 'ewd-premium-helper' );			
		}

		if ( fdm_ultimate_active() ) {

			$messages[] = ! empty( get_option( 'fdm-license-key', false ) ) ? $this->add_notification_token( $token, 'fdm', get_option( 'fdm-license-key' ) ) : __( 'Invalid Restaurant Menu Ultimate license key.', 'ewd-premium-helper' );
		}

		$messages = array_unique( $messages );
			
		if ( sizeof( $messages ) > 1 or reset( $messages ) != __( 'You have been subscribed for push notifications.', 'ewd-premium-helper' ) ) {

			return array( 'success' => false, 'message' => implode( '\n', $messages ) );
		}

		do_action( 'fspph_new_device_token_saved' );

		return array( 'success' => true, 'message' => reset( $messages ) );
	}

	/**
	 * Save token to database sent from app
	 * */
	protected function add_notification_token( $token, $plugin, $license_key ) {
		
		$url = add_query_arg(
			array(
				'license_key' => urlencode( $license_key ),
				'plugin'      => urlencode( $plugin ),
				'token_list'  => urlencode( $token )
			),
			'https://www.etoilewebdesign.com/pushnotihandler/save-device-token.php'
		);

		$response = wp_remote_get( $url );
		fspph_debug( 'response ' . print_r( $response, true ) );

		if ( $response instanceof WP_Error ) {

			$message = $response->has_errors() ? $response->get_error_message() : __( 'The push notification token did not process properly.', 'ewd-premium-helper' );

			$this->add_error_log_entry( 'TKN001', $message );

			return $message;
		}
		
		$body = json_decode( wp_remote_retrieve_body( $response ), true );
					
		if ( ! $body['success'] ) {

			$error_code = isset( $body['code'] ) ? $body['code'] : __( 'Unknown Code', 'ewd-premium-helper' );
			$message = ! empty( $body['message'] ) ? $body['message'] : __( 'The token did not process properly. If you keep getting this error, contact Five Star Plugins support at contact@fivestarplugins.com.', 'ewd-premium-helper' );
						
			$this->add_error_log_entry( $error_code, $message );

			return $message;
		}
		
		return __( 'You have been subscribed for push notifications.', 'ewd-premium-helper' );
	}

	/**
	 * Return a message when an API request isn't accompanied by a valid key
	 * @since 1.0.0
	 */
	public function return_invalid_api_key() {

		return array( 'success' => false, 'message' => __( 'The API key sent with your request was not valid.', 'ewd-premium-helper' ) );
	}

	/**
	 * Return a message when a push notification save request doesn't have the ultimate version active
	 * @since 1.0.0
	 */
	public function return_no_ultimate_active() {

		return array( 'success' => false, 'message' => __( 'The API key sent with your request is not associated with a valid Ultimate plugin license.', 'ewd-premium-helper' ) );
	}

	/**
	 * Return a message when an API request doesn't have updating privileges
	 * @since 1.0.0
	 */
	public function return_insufficient_api_key_privileges() {

		return array( 'success' => false, 'message' => __( 'The API key sent with your request does not having updating privileges.', 'ewd-premium-helper' ) );
	}

	/**
	 * Add the API key settings panel
	 * @since 1.0.0
	 */
	public function add_plugin_settings_panel( $sap ) {
		global $ewd_otp_controller;
		
		if ( ! $ewd_otp_controller->permissions->check_permission( 'api_usage' ) ) {
			$api_permissions = array(
				'disabled'        => true,
				'disabled_image'  => '#',
				'purchase_link'   => 'https://www.etoilewebdesign.com/plugins/order-tracking/',
				'ultimate_needed' => true,
			);
		}
		else { $api_permissions = array(); }

		$sap->add_section(
			'ewd-otp-settings',
			array(
				'id'            => 'ewd-otp-api-tab',
				'title'         => __( 'API', 'ewd-premium-helper' ),
				'is_tab'		=> true,
				'rank'			=> 99
			)
		);

		$sap->add_section(
			'ewd-otp-settings',
			array_merge( 
				array(
					'id'    => 'ewd-otp-api',
					'title' => __( 'App Access Codes', 'ewd-premium-helper' ),
					'tab'	=> 'ewd-otp-api-tab',
				),
				$api_permissions
			)
		);

		$sap->add_setting(
			'ewd-otp-settings',
			'ewd-otp-api',
			'infinite_table',
			array(
				'id'			=> 'ewd-api-keys',
				'title'			=> __( 'App API Keys', 'ewd-premium-helper' ),
				'add_label'		=> __( 'Add Key', 'ewd-premium-helper' ),
				'del_label'		=> __( 'Delete', 'ewd-premium-helper' ),
				'description'	=> __( '', 'ewd-premium-helper' ),
				'fields'		=> array(
					'api_key' => array(
						'type' 		=> 'text',
						'label' 	=> 'Key',
						'required' 	=> true
					),
					'access' => array(
						'type' 		=> 'select',
						'label'		=> 'Access',
						'required'	=> true,
						'options' 	=> array(
							'view' 		=> 'View',
							'update' 	=> 'Update',
						)
					)
				)
			)
		);

		$sap->add_setting(
			'ewd-otp-settings',
			'ewd-otp-api',
			'html',
			array(
				'id'			=> 'otpm-error-messages',
				'title'			=> __( 'Order Tracking Manager Error Log', 'ewd-premium-helper' ),
				'html'			=> '
					<p class="description">' . __( 'The table below holds the 10 most recent error notifications that are less than a week old.', 'ewd-premium-helper' ) . '</p>' .
					$this->render_error_log_table(),
			)
		);

		return $sap;
	}

	/**
	 * Delete OTPM error messages older than a week, then display a table with up to 10 messages
	 * @since 0.0.16
	 */
	public function render_error_log_table() {

		$error_messages = is_array( get_option( 'otpm-error-log' ) ) ? get_option( 'otpm-error-log' ) : array();

		foreach ( $error_messages as $key => $error_message ) {

			if ( $error_message['timestamp'] + 7*24*3600 < time() ) { unset( $error_messages[ $key ] ); }
		}

		update_option( 'otpm-error-log', $error_messages );

		if ( empty( $error_messages ) ) {

			return '<p>' . __( 'No error messages to display', 'ewd-premium-helper' ) . '</p>';
		}

		ob_start();

		?>

		<table>

			<thead>
				<tr>
					<th><?php _e( 'Date/Time', 'ewd-premium-helper' ); ?></th>
					<th><?php _e( 'Error Code', 'ewd-premium-helper' ); ?></th>
					<th><?php _e( 'Message', 'ewd-premium-helper' ); ?></th>
				</tr>
			</thead>

			<tbody>

				<?php foreach ( $error_messages as $error_message ) { ?>

					<tr>
						<td><?php echo date( 'Y-m-d H:i:s', $error_message['timestamp'] ); ?></td>
						<td><?php echo esc_html( $error_message['error_code'] ); ?></td>
						<td><?php echo esc_html( $error_message['message'] ); ?></td>
					</tr>

				<?php } ?>

			</tbody>

		</table>

		<?php 

		return ob_get_clean();
	}

	/**
	 * Adds an entry to the error log option
	 * @since 0.0.16
	 */
	public function add_error_log_entry( $error_code, $message ) {
		  
		$error_messages = is_array( get_option( 'otpm-error-log' ) ) ? get_option( 'otpm-error-log' ) : array();

		$error_messages[] = array(
			'timestamp'		=> time(),
			'error_code'	=> $error_code,
			'message'		=> $message,
		);

		usort( $error_messages, array( $this, 'sort_by_timestamp' ) );
		 
		update_option( 'otpm-error-log', array_slice( $error_messages, 0, 10 ) );
	}

	/**
	 * Adds an entry to the error log option
	 * @since 0.0.16
	 */
	public function sort_by_timestamp( $a, $b ) {

		return $b['timestamp'] <=> $a['timestamp'];
	}

	/**
	 * Add in the JS to create a random 20 character key on 'Add' click
	 * @since 1.0.0
	 */
	public function enqueue_admin_assets( $hook ) {
		
		if ( $hook != 'order-tracking_page_ewd-otp-settings' ) { return; }

		wp_enqueue_script( 'ewd-admin-js', EWDPH_PLUGIN_URL . '/assets/js/ewd-admin.js', array( 'jquery' ), EWDPH_VERSION, true );
	}
}
}

?>